# Copyright 2018 Adobe. All rights reserved.

import os
import shutil
import sys
import tempfile
import unittest
from io import StringIO

from opentypesvg import utils as shared_utils


class SharedUtilsTest(unittest.TestCase):

    @staticmethod
    def write_binary_file(file_path, data):
        with open(file_path, "wb") as f:
            f.write(data)

    @staticmethod
    def reset_stream(stream):
        stream.seek(0)
        stream.truncate(0)
        return stream

# -----
# Tests
# -----

    def test_read_write_file(self):
        content = '1st line\n2nd line\n3rd line'
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            shared_utils.write_file(tmp.name, content)
            result = shared_utils.read_file(tmp.name)
            self.assertEqual(result.splitlines()[1], '2nd line')

    def test_get_font_format(self):
        in_out = {"OTTO": "OTF",
                  "\x00\x01\x00\x00": "TTF",
                  "true": "TTF",
                  "wOFF": "WOFF",
                  "wOF2": "WOFF2",
                  "blah": None}
        with tempfile.NamedTemporaryFile(delete=False) as tmp:
            for key, val in in_out.items():
                data = bytearray(key, encoding="utf-8")
                self.write_binary_file(tmp.name, data)
                result = shared_utils.get_font_format(tmp.name)
                self.assertEqual(result, val)

    def test_final_message(self):
        stream = sys.stdout = StringIO()
        shared_utils.final_message(0)
        self.assertEqual(stream.getvalue().strip(), 'No SVG files saved.')

        self.reset_stream(stream)
        shared_utils.final_message(1)
        self.assertEqual(stream.getvalue().strip(), '1 SVG file saved.')

        self.reset_stream(stream)
        shared_utils.final_message(2)
        self.assertEqual(stream.getvalue().strip(), '2 SVG files saved.')

    def test_create_folder(self):
        folder_path = 'new_folder'
        shared_utils.create_folder(folder_path)
        self.assertTrue(os.path.isdir(folder_path))

        # change just-created folder to be unaccessible
        os.chmod(folder_path, 0o0)

        # try creating a folder inside the unaccessible one
        # NOTE: apparently there's no way to make a Windows folder
        # read-only programmatically, so first check if the parent
        # folder cannot be accessed. See Windows note in os.chmod()
        # https://docs.python.org/3/library/os.html#os.chmod
        blocked_folder_path = os.path.join(folder_path, 'blocked_folder')
        if not os.access(folder_path, os.W_OK):
            with self.assertRaises(OSError) as cm:
                shared_utils.create_folder(blocked_folder_path)
            self.assertEqual(cm.exception.errno, 13)

        # try creating a folder with the same name as an existing file
        file_path = 'new_file'
        with open(file_path, "w+"):
            with self.assertRaises(OSError) as cm:
                shared_utils.create_folder(file_path)
            self.assertEqual(cm.exception.errno, 17)

        # remove artifacts
        os.chmod(folder_path, 0o755)
        if os.path.exists(folder_path):
            os.rmdir(folder_path)
        if os.path.exists(file_path):
            os.remove(file_path)

    def test_create_nested_folder(self):
        nested_folder_path = 'folder_path'
        result = shared_utils.create_nested_folder(nested_folder_path, None)
        self.assertEqual(result, nested_folder_path)

        main_folder_path = 'main_folder'
        nested_folder_path = os.path.join(main_folder_path,
                                          shared_utils.NESTED_FOLDER_NAME)
        shared_utils.create_nested_folder(None, main_folder_path)
        self.assertTrue(os.path.isdir(nested_folder_path))
        # remove artifact
        if os.path.exists(nested_folder_path):
            shutil.rmtree(main_folder_path)

    def test_get_output_folder_path(self):
        folder_path = 'fonts'
        result = shared_utils.get_output_folder_path(folder_path, None)
        self.assertEqual(os.path.basename(result), folder_path)

        font_path = 'font'
        result = shared_utils.get_output_folder_path(None, font_path)
        self.assertEqual(os.path.basename(result),
                         shared_utils.SVG_FOLDER_NAME)

    def test_get_gnames_to_save_in_nested_folder(self):
        result = shared_utils.get_gnames_to_save_in_nested_folder(
            ['a', 'A', 'B', 'b', 'c'])
        self.assertEqual(result, ['A', 'b'])


if __name__ == "__main__":
    sys.exit(unittest.main())



#!/usr/bin/env python

# Copyright 2016 Adobe. All rights reserved.

"""
Generates a set of SVG glyph files from one or more fonts and hex colors
for each of them. The fonts' format can be either OpenType, TrueType, WOFF,
or WOFF2.
"""

import argparse
import os
import re
import sys

from fontTools import ttLib
from fontTools.pens.basePen import BasePen
from fontTools.pens.transformPen import TransformPen

from opentypesvg.__version__ import version as __version__
from opentypesvg.utils import (
    create_folder,
    create_nested_folder,
    final_message,
    get_gnames_to_save_in_nested_folder,
    get_output_folder_path,
    split_comma_sequence,
    validate_font_paths,
    write_file,
)


class SVGPen(BasePen):

    def __init__(self, glyphSet):
        BasePen.__init__(self, glyphSet)
        self.d = u''
        self._lastX = self._lastY = None

    def _moveTo(self, pt):
        ptx, pty = self._isInt(pt)
        self.d += u'M{} {}'.format(ptx, pty)
        self._lastX, self._lastY = pt

    def _lineTo(self, pt):
        ptx, pty = self._isInt(pt)
        if (ptx, pty) == (self._lastX, self._lastY):
            return
        elif ptx == self._lastX:
            self.d += u'V{}'.format(pty)
        elif pty == self._lastY:
            self.d += u'H{}'.format(ptx)
        else:
            self.d += u'L{} {}'.format(ptx, pty)
        self._lastX, self._lastY = pt

    def _curveToOne(self, pt1, pt2, pt3):
        pt1x, pt1y = self._isInt(pt1)
        pt2x, pt2y = self._isInt(pt2)
        pt3x, pt3y = self._isInt(pt3)
        self.d += u'C{} {} {} {} {} {}'.format(pt1x, pt1y, pt2x, pt2y,
                                               pt3x, pt3y)
        self._lastX, self._lastY = pt3

    def _qCurveToOne(self, pt1, pt2):
        pt1x, pt1y = self._isInt(pt1)
        pt2x, pt2y = self._isInt(pt2)
        self.d += u'Q{} {} {} {}'.format(pt1x, pt1y, pt2x, pt2y)
        self._lastX, self._lastY = pt2

    def _closePath(self):
        self.d += u'Z'
        self._lastX = self._lastY = None

    def _endPath(self):
        self._closePath()

    @staticmethod
    def _isInt(tup):
        return [int(flt) if (flt).is_integer() else flt for flt in tup]


def processFonts(font_paths_list, hex_colors_list, outputFolderPath, options):
    glyphSetsList = []
    allGlyphNamesList = []

    # Load the fonts and collect their glyph sets
    for fontPath in font_paths_list:
        font = ttLib.TTFont(fontPath)
        gSet = font.getGlyphSet()
        glyphSetsList.append(gSet)
        allGlyphNamesList.append(gSet.keys())
        font.close()

    if not glyphSetsList:
        raise AssertionError("No glyph sets.")

    # Define the list of glyph names to convert to SVG
    if options.gnames_to_generate:
        glyphNamesList = sorted(options.gnames_to_generate)
    else:
        if options.glyphsets_union:
            glyphNamesList = sorted(
                list(set.union(*map(set, allGlyphNamesList))))
        else:
            glyphNamesList = sorted(
                list(set.intersection(*map(set, allGlyphNamesList))))
            # Extend the list with additional glyph names
            if options.gnames_to_add:
                glyphNamesList.extend(options.gnames_to_add)
                # Remove any duplicates and sort
                glyphNamesList = sorted(list(set(glyphNamesList)))

    # Confirm that there's something to process
    if not glyphNamesList:
        print("The fonts and options provided can't produce any SVG files.",
              file=sys.stdout)
        return

    # Define the list of glyph names to skip
    glyphNamesToSkipList = [".notdef"]
    if options.gnames_to_exclude:
        glyphNamesToSkipList.extend(options.gnames_to_exclude)

    # Determine which glyph names need to be saved in a nested folder
    glyphNamesToSaveInNestedFolder = get_gnames_to_save_in_nested_folder(
        glyphNamesList)

    nestedFolderPath = None
    filesSaved = 0

    viewbox = viewbox_settings(
        font_paths_list[0],
        options.adjust_view_box_to_glyph
    )

    # Generate the SVGs
    for gName in glyphNamesList:
        svgStr = (u"""<svg xmlns="http://www.w3.org/2000/svg" """
                  u"""viewBox="{}">\n""".format(viewbox))

        for index, gSet in enumerate(glyphSetsList):
            # Skip glyphs that don't exist in the current font,
            # or that were requested to be skipped
            if gName not in gSet.keys() or gName in glyphNamesToSkipList:
                continue

            pen = SVGPen(gSet)
            tpen = TransformPen(pen, (1.0, 0.0, 0.0, -1.0, 0.0, 0.0))
            glyph = gSet[gName]
            glyph.draw(tpen)
            d = pen.d
            # Skip glyphs with no contours
            if not len(d):
                continue

            hex_str = hex_colors_list[index]
            opc = ''
            if len(hex_str) != 6:
                opcHex = hex_str[6:]
                hex_str = hex_str[:6]
                if opcHex.lower() != 'ff':
                    opc = ' opacity="{:.2f}"'.format(int(opcHex, 16) / 255)

            svgStr += u'\t<path{} fill="#{}" d="{}"/>\n'.format(
                opc, hex_str, d)
        svgStr += u'</svg>'

        # Skip saving files that have no paths
        if '<path' not in svgStr:
            continue

        # Create the output folder.
        # This may be necessary if the folder was not provided.
        # The folder is made this late in the process because
        # only now it's clear that's needed.
        create_folder(outputFolderPath)

        # Create the nested folder, if there are conflicting glyph names.
        if gName in glyphNamesToSaveInNestedFolder:
            folderPath = create_nested_folder(nestedFolderPath,
                                              outputFolderPath)
        else:
            folderPath = outputFolderPath

        svgFilePath = os.path.join(folderPath, gName + '.svg')
        write_file(svgFilePath, svgStr)
        filesSaved += 1

    font.close()
    final_message(filesSaved)


def viewbox_settings(font_path, adjust_view_box_to_glyph):
    try:
        head = ttLib.TTFont(font_path)["head"]
        if adjust_view_box_to_glyph:
            # it looks like compared to viewbox in the head table
            # the yMin and yMax are inverted
            x = head.xMin
            y = -head.yMax
            width = head.xMax - head.xMin
            height = head.yMax - head.yMin
            return """{} {} {} {}""".format(x, y, width, height)
        else:
            # Gather the fonts' UPM. For simplicity,
            # it's assumed that all fonts have the same UPM value.
            # If fetching the UPM value fails, default to 1000.
            upm = head.unitsPerEm
            return """0 -{} {} {}""".format(upm, upm, upm)
    except KeyError:
        upm = 1000
        return """0 -{} {} {}""".format(upm, upm, upm)


RE_HEXCOLOR = re.compile(r"^(?=[a-fA-F0-9]*$)(?:.{6}|.{8})$")


def validate_hex_values(hex_str):
    hex_values = split_comma_sequence(hex_str)
    for hex_val in hex_values:
        if not RE_HEXCOLOR.match(hex_val):
            raise argparse.ArgumentTypeError(
                "{} is not a valid hex color.".format(hex_val))
    return hex_values


def get_options(args):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        description=__doc__
    )
    parser.add_argument(
        '--version',
        action='version',
        version=__version__
    )
    parser.add_argument(
        '-c',
        metavar='HEX_VALUES',
        dest='colors_list',
        type=validate_hex_values,
        default=[],
        help='comma-separated list of hex colors in RRGGBBAA format.\n'
             'The alpha value (AA) is optional.'
    )
    parser.add_argument(
        '-o',
        metavar='FOLDER_PATH',
        dest='output_folder_path',
        help='path to folder for outputting the SVG files to.'
    )
    parser.add_argument(
        '-g',
        metavar='GLYPH_NAMES',
        dest='gnames_to_generate',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to make SVG files from.'
    )
    parser.add_argument(
        '-a',
        metavar='GLYPH_NAMES',
        dest='gnames_to_add',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to add.'
    )
    parser.add_argument(
        '-x',
        metavar='GLYPH_NAMES',
        dest='gnames_to_exclude',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to exclude.'
    )
    parser.add_argument(
        '-u',
        action='store_true',
        dest='glyphsets_union',
        help="do union (instead of intersection) of the fonts' glyph sets."
    )
    parser.add_argument(
        '-av', '--adjust-viewbox',
        action='store_true',
        dest='adjust_view_box_to_glyph',
        help="vertically center the viewBox on the bounding box of all glyphs."
    )
    parser.add_argument(
        'input_paths',
        metavar='FONT',
        nargs='+',
        help='OTF/TTF/WOFF/WOFF2 font file.',
    )
    options = parser.parse_args(args)

    options.font_paths_list = validate_font_paths(options.input_paths)
    return options


def main(args=None):
    opts = get_options(args)

    if not opts.font_paths_list:
        return 1

    font_paths_list = opts.font_paths_list
    hex_colors_list = opts.colors_list

    # Confirm that the number of colors is the same as the fonts. If it's not,
    # extend the list of colors using SVG's default color (black), or trim the
    # list of colors.
    if len(hex_colors_list) < len(font_paths_list):
        num_add_col = len(font_paths_list) - len(hex_colors_list)
        hex_colors_list.extend(['000000'] * num_add_col)
        print("WARNING: The list of colors was extended with {} #000000 "
              "value(s).".format(num_add_col), file=sys.stderr)
    elif len(hex_colors_list) > len(font_paths_list):
        num_xtr_col = len(hex_colors_list) - len(font_paths_list)
        del hex_colors_list[len(font_paths_list):]
        print("WARNING: The list of colors got the last {} value(s) truncated:"
              " {}".format(num_xtr_col, ' '.join(
                  hex_colors_list[-num_xtr_col:])), file=sys.stderr)

    output_folder_path = get_output_folder_path(opts.output_folder_path,
                                                font_paths_list[0])

    processFonts(font_paths_list, hex_colors_list, output_folder_path, opts)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

# Copyright 2016 Adobe. All rights reserved.

"""
Adds an SVG table to a font, using SVG files provided.
The font format can be either OpenType or TrueType.
"""

import argparse
import os
import re
import sys
from shutil import copy2

from fontTools import ttLib

from opentypesvg.__version__ import version as __version__
from opentypesvg.utils import (
    read_file,
    split_comma_sequence,
    validate_folder_path,
    validate_font_paths,
)


def getGlyphNameFromFileName(filePath):
    fontFileName = os.path.split(filePath)[1]
    return os.path.splitext(fontFileName)[0]


reIDvalue = re.compile(r"<svg[^>]+?(id=\".*?\").+?>", re.DOTALL)


def setIDvalue(data, gid):
    id_value = reIDvalue.search(data)
    if id_value:
        return re.sub(id_value.group(1), 'id="glyph{}"'.format(gid), data)
    return re.sub('<svg', '<svg id="glyph{}"'.format(gid), data)


# The value of the viewBox attribute is a list of four numbers min-x, min-y,
# width and height, separated by whitespace and/or a comma
reViewBox = re.compile(
    r"(<svg.+?)(\s*viewBox=[\"|\'](?:[-\d,. ]+)[\"|\'])(.+?>)", re.DOTALL)


def stripViewBox(svgItemData):
    """
    Removes the viewBox parameter from the <svg> element.
    """
    vb = reViewBox.search(svgItemData)
    if vb:
        svgItemData = reViewBox.sub(r"\g<1>\g<3>", svgItemData)
    return svgItemData


reXMLheader = re.compile(r"<\?xml .*\?>")
reEnableBkgrd = re.compile(r"( enable-background=[\"|\'][new\d, ]+[\"|\'])")
reWhiteSpaceBtween = re.compile(r">\s+<", re.MULTILINE)
reWhiteSpaceWithin = re.compile(r"\s+", re.MULTILINE)


def cleanupSVGdoc(svgItemData):
    # Remove XML header
    svgItemData = reXMLheader.sub('', svgItemData)

    # Remove all 'enable-background' parameters
    for enableBkgrd in reEnableBkgrd.findall(svgItemData):
        svgItemData = svgItemData.replace(enableBkgrd, '')

    # Remove all white space BETWEEN elements
    for whiteSpace in reWhiteSpaceBtween.findall(svgItemData):
        svgItemData = svgItemData.replace(whiteSpace, '><')

    # Replace all white space WITHIN elements with a single space
    for whiteSpace2 in reWhiteSpaceWithin.findall(svgItemData):
        svgItemData = svgItemData.replace(whiteSpace2, ' ')

    return svgItemData


reCopyCounter = re.compile(r"#\d+$")


def makeFontCopyPath(fontPath):
    dirName, fileName = os.path.split(fontPath)
    fileName, fileExt = os.path.splitext(fileName)
    fileName = reCopyCounter.split(fileName)[0]
    fontCopyPath = os.path.join(dirName, fileName + fileExt)
    n = 0
    while os.path.exists(fontCopyPath):
        newPath = fileName + "#" + repr(n) + fileExt
        fontCopyPath = os.path.join(dirName, newPath)
        n += 1
    return fontCopyPath


def processFont(fontPath, svgFilePathsList, options):
    font = ttLib.TTFont(fontPath)

    svgDocsDict = {}
    gNamesSeenAlreadyList = []

    svgGlyphsAdded = 0

    for svgFilePath in svgFilePathsList:
        gName = getGlyphNameFromFileName(svgFilePath)

        if gName in options.gnames_to_exclude:
            continue

        try:
            gid = font.getGlyphID(gName)
        except KeyError:
            print("WARNING: Could not find a glyph named {} in the font "
                  "{}".format(gName, os.path.split(fontPath)[1]),
                  file=sys.stderr)
            continue

        if gName in gNamesSeenAlreadyList:
            print("WARNING: Skipped a duplicate file named {}.svg at "
                  "{}".format(gName, svgFilePath), file=sys.stderr)
            continue
        else:
            gNamesSeenAlreadyList.append(gName)

        svgItemData = read_file(svgFilePath)

        # Set id value
        svgItemData = setIDvalue(svgItemData, gid)

        # Remove the viewBox parameter
        if options.strip_viewbox:
            svgItemData = stripViewBox(svgItemData)

        # Clean-up SVG document
        svgItemData = cleanupSVGdoc(svgItemData)

        svgDocsDict[gid] = [svgItemData.strip(), gid, gid]
        svgGlyphsAdded += 1

    # Don't do any changes to the input font if there's no SVG data
    if not svgDocsDict:
        print("Could not find any SVG files that can be added to the font.",
              file=sys.stdout)
        sys.exit(0)

    svgDocsList = [svgDocsDict[index] for index in sorted(svgDocsDict.keys())]

    svgTable = ttLib.newTable('SVG ')
    svgTable.compressed = options.compress_svgs
    svgTable.docList = svgDocsList
    svgTable.colorPalettes = None
    font['SVG '] = svgTable

    # Make copy of the original font
    if options.make_font_copy:
        fontCopyPath = makeFontCopyPath(fontPath)
        copy2(fontPath, fontCopyPath)

    font.save(fontPath)

    if options.generate_woffs:
        # WOFF files are smaller if SVG table is uncompressed
        font['SVG '].compressed = False
        for ext in ['woff', 'woff2']:
            woffFontPath = os.path.splitext(fontPath)[0] + '.' + ext
            font.flavor = ext
            font.save(woffFontPath)

    font.close()

    plural = 's were' if svgGlyphsAdded != 1 else ' was'
    print("{} SVG glyph{} successfully added to {}".format(
        svgGlyphsAdded, plural, os.path.split(fontPath)[1]), file=sys.stdout)


reSVGelement = re.compile(r"<svg.+?>.+?</svg>", re.DOTALL)
reTEXTelement = re.compile(r"<text.+?>.+?</text>", re.DOTALL)


def validateSVGfiles(svgFilePathsList):
    """
    Light validation of SVG files.
      - checks that there is an <svg> element.
      - skips files that have a <text> element.
    """
    validatedPaths = []

    for filePath in svgFilePathsList:
        # Skip hidden files (filenames that start with period)
        fileName = os.path.basename(filePath)
        if fileName.startswith('.'):
            continue

        # Skip files that don't end with SVG extension
        if not fileName.lower().endswith('.svg'):
            continue

        if not os.path.isfile(filePath):
            raise AssertionError("Not a valid file path: {}".format(filePath))
        data = read_file(filePath)

        # Find <svg> blob
        svg = reSVGelement.search(data)
        if not svg:
            print("WARNING: Could not find <svg> element in the file. "
                  "Skiping {}".format(filePath))
            continue

        # Warn about <text> elements
        text = reTEXTelement.search(data)
        if text:
            print("WARNING: Found <text> element in the file. "
                  "Skiping {}".format(filePath))
            continue

        validatedPaths.append(filePath)

    return validatedPaths


def get_options(args):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        description=__doc__
    )
    parser.add_argument(
        '--version',
        action='version',
        version=__version__
    )
    parser.add_argument(
        '-m',
        action='store_false',
        dest='make_font_copy',
        help='do not make a copy of the input font.'
    )
    parser.add_argument(
        '-k',
        action='store_false',
        dest='strip_viewbox',
        help="do not strip the 'viewBox' parameter."
    )
    parser.add_argument(
        '-w',
        action='store_true',
        dest='generate_woffs',
        help='generate WOFF and WOFF2 formats.'
    )
    parser.add_argument(
        '-x',
        metavar='GLYPH_NAMES',
        dest='gnames_to_exclude',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to exclude.'
    )
    parser.add_argument(
        '-z',
        action='store_true',
        dest='compress_svgs',
        help='compress the SVG table.'
    )
    parser.add_argument(
        'svg_folder_path',
        metavar='FOLDER_PATH',
        type=validate_folder_path,
        help='path to folder containing SVG files.\n'
             'The file names MUST match the names of the\n'
             "glyphs they're meant to be associated with."
    )
    parser.add_argument(
        'input_path',
        metavar='FONT',
        help='OTF/TTF font file.',
    )
    options = parser.parse_args(args)

    options.font_paths_list = validate_font_paths([options.input_path])
    return options


def main(args=None):
    opts = get_options(args)

    if not opts.font_paths_list:
        return 1

    # Collect the paths to SVG files
    svgFilePathsList = []
    for dirName, _, fileList in os.walk(opts.svg_folder_path):
        # Support nested folders
        for file in fileList:
            svgFilePathsList.append(os.path.join(dirName, file))

    # Validate the SVGs
    svgFilePathsList = validateSVGfiles(svgFilePathsList)

    if not svgFilePathsList:
        print("No SVG files were found.", file=sys.stdout)
        return 1

    processFont(opts.font_paths_list[0], svgFilePathsList, opts)


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2021 Adobe. All rights reserved.

import os
import pytest


@pytest.fixture
def fonts_dir():
    root_dir = os.path.dirname(os.path.dirname(__file__))
    yield os.path.join(root_dir, 'fonts')


@pytest.fixture
def base_font_path(fonts_dir):
    yield os.path.join(fonts_dir, 'Zebrawood.otf')


@pytest.fixture
def fixtures_dir():
    yield os.path.join(os.path.dirname(__file__), 'fixtures')



# Copyright 2016 Adobe. All rights reserved.

"""
Module that contains shared functionality.
"""

import os
import sys


SVG_FOLDER_NAME = "SVGs"
NESTED_FOLDER_NAME = "_moreSVGs_"


def read_file(file_path):
    with open(file_path, "r") as f:
        return f.read()


def write_file(file_path, data):
    with open(file_path, "w") as f:
        f.write(data)


def get_font_format(font_file_path):
    with open(font_file_path, "rb") as f:
        head = f.read(4).decode()

    if head == "OTTO":
        return "OTF"
    elif head in ("\x00\x01\x00\x00", "true"):
        return "TTF"
    elif head == "wOFF":
        return "WOFF"
    elif head == "wOF2":
        return "WOFF2"
    return None


def validate_font_paths(paths_list):
    validated_paths_list = []
    for path in paths_list:
        path = os.path.realpath(path)
        if (os.path.isfile(path) and get_font_format(path) in
                ['OTF', 'TTF', 'WOFF', 'WOFF2']):
            validated_paths_list.append(path)
        else:
            print("ERROR: {} is not a valid font file path.".format(path),
                  file=sys.stderr)
    return validated_paths_list


def split_comma_sequence(comma_str):
    return [item.strip() for item in comma_str.split(',')]


def final_message(num_files_saved):
    if not num_files_saved:
        num_files_saved = 'No'
    plural = 's' if num_files_saved != 1 else ''
    print("{} SVG file{} saved.".format(num_files_saved, plural),
          file=sys.stdout)


def create_folder(folder_path):
    try:
        os.makedirs(folder_path)
    except OSError:
        if not os.path.isdir(folder_path):
            raise


def create_nested_folder(nested_folder_path, main_folder_path):
    """
    Creates a nested folder and returns its path.
    This additional folder is created when file names conflict.
    """
    if not nested_folder_path:
        nested_folder_path = os.path.join(main_folder_path, NESTED_FOLDER_NAME)
        create_folder(nested_folder_path)
    return nested_folder_path


def validate_folder_path(folder_path):
    """
    Validates that the path is a folder.
    Returns the complete path.
    """
    path = os.path.realpath(folder_path)
    if os.path.isdir(path):
        return path
    else:
        print("ERROR: {} is not a valid folder path.".format(path),
              file=sys.stderr)
        sys.exit(1)


def get_output_folder_path(provided_folder_path, first_font_path):
    """
    If the path to the output folder was NOT provided, create
    a folder in the same directory where the first font is.
    If the path was provided, validate it.
    Returns a valid output folder.
    """
    if provided_folder_path:
        return validate_folder_path(provided_folder_path)
    return os.path.join(os.path.dirname(first_font_path), SVG_FOLDER_NAME)


def get_gnames_to_save_in_nested_folder(gnames_list):
    """
    On case-insensitive systems the SVG files cannot be all saved to the
    same folder otherwise a.svg and A.svg would be written over each other,
    for example. So, pre-process the list of glyph names to find which ones
    step on each other, and save half of them in a nested folder. This
    approach won't handle the case where a.svg and A.svg are NOT generated
    on the same run, but that's fine; the user will have to handle that.
    Also, the process below assumes that there are no more than 2 conflicts
    per name, i.e. it will handle "the/The" but not "the/The/THE/...";
    this shouldn't be a problem in 99% of the time.
    Returns list of glyph names that need to be saved in a nested folder.
    """
    unique_names_set = set()
    gnames_to_save_in_nested_folder = []
    for gname in gnames_list:
        if gname.lower() in unique_names_set:
            gnames_to_save_in_nested_folder.append(gname)
        unique_names_set.add(gname.lower())
    return gnames_to_save_in_nested_folder



#!/usr/bin/env python

# Copyright 2016 Adobe. All rights reserved.

"""
Saves the contents of a font's SVG table as individual SVG files.
The font's format can be either OpenType, TrueType, WOFF, or WOFF2.
"""

import argparse
import os
import re
import sys

from fontTools import ttLib

from opentypesvg.__version__ import version as __version__
from opentypesvg.utils import (
    create_folder,
    create_nested_folder,
    final_message,
    get_gnames_to_save_in_nested_folder,
    get_output_folder_path,
    split_comma_sequence,
    validate_font_paths,
    write_file,
)


reViewBox = re.compile(r"viewBox=[\"|\']([\d, ])+?[\"|\']", re.DOTALL)


def resetViewBox(svgDoc):
    viewBox = reViewBox.search(svgDoc)
    if not viewBox:
        return svgDoc
    viewBoxPartsList = viewBox.group().split()
    viewBoxPartsList[1] = 0
    replacement = ' '.join(map(str, viewBoxPartsList))
    svgDocChanged = re.sub(viewBox.group(), replacement, svgDoc)
    return svgDocChanged


def processFont(fontPath, outputFolderPath, options):
    font = ttLib.TTFont(fontPath)
    glyphOrder = font.getGlyphOrder()
    svgTag = 'SVG '

    if svgTag not in font:
        print("ERROR: The font does not have the {} table.".format(
            svgTag.strip()), file=sys.stderr)
        sys.exit(1)

    svgTable = font[svgTag]

    if not len(svgTable.docList):
        print("ERROR: The {} table has no data that can be output.".format(
            svgTag.strip()), file=sys.stderr)
        sys.exit(1)

    # Define the list of glyph names to convert to SVG
    if options.gnames_to_generate:
        glyphNamesList = sorted(options.gnames_to_generate)
    else:
        glyphNamesList = sorted(glyphOrder)

    # Confirm that there's something to process
    if not glyphNamesList:
        print("The fonts and options provided can't produce any SVG files.",
              file=sys.stdout)
        return

    # Define the list of glyph names to skip
    glyphNamesToSkipList = [".notdef"]
    if options.gnames_to_exclude:
        glyphNamesToSkipList.extend(options.gnames_to_exclude)

    # Determine which glyph names need to be saved in a nested folder
    glyphNamesToSaveInNestedFolder = get_gnames_to_save_in_nested_folder(
        glyphNamesList)

    nestedFolderPath = None
    filesSaved = 0
    unnamedNum = 1

    # Write the SVG files by iterating over the entries in the SVG table.
    # The process assumes that each SVG document contains only one 'id' value,
    # i.e. the artwork for two or more glyphs is not included in a single SVG.
    for svgItemsList in svgTable.docList:
        svgItemData, startGID, endGID = svgItemsList

        if options.reset_viewbox:
            svgItemData = resetViewBox(svgItemData)

        while(startGID != endGID + 1):
            try:
                gName = glyphOrder[startGID]
            except IndexError:
                gName = "_unnamed{}".format(unnamedNum)
                glyphNamesList.append(gName)
                unnamedNum += 1
                print("WARNING: The SVG table references a glyph ID (#{}) "
                      "which the font does not contain.\n"
                      "         The artwork will be saved as '{}.svg'.".format(
                          startGID, gName), file=sys.stderr)

            if gName not in glyphNamesList or gName in glyphNamesToSkipList:
                startGID += 1
                continue

            # Create the output folder.
            # This may be necessary if the folder was not provided.
            # The folder is made this late in the process because
            # only now it's clear that's needed.
            create_folder(outputFolderPath)

            # Create the nested folder, if there are conflicting glyph names.
            if gName in glyphNamesToSaveInNestedFolder:
                folderPath = create_nested_folder(nestedFolderPath,
                                                  outputFolderPath)
            else:
                folderPath = outputFolderPath

            svgFilePath = os.path.join(folderPath, gName + '.svg')
            write_file(svgFilePath, svgItemData)
            filesSaved += 1
            startGID += 1

    font.close()
    final_message(filesSaved)


def get_options(args):
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawTextHelpFormatter,
        description=__doc__
    )
    parser.add_argument(
        '--version',
        action='version',
        version=__version__
    )
    parser.add_argument(
        '-o',
        metavar='FOLDER_PATH',
        dest='output_folder_path',
        help='path to folder for outputting the SVG files to.'
    )
    parser.add_argument(
        '-r',
        action='store_true',
        dest='reset_viewbox',
        help='reset viewBox values.'
    )
    parser.add_argument(
        '-g',
        metavar='GLYPH_NAMES',
        dest='gnames_to_generate',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to make SVG files from.'
    )
    parser.add_argument(
        '-x',
        metavar='GLYPH_NAMES',
        dest='gnames_to_exclude',
        type=split_comma_sequence,
        default=[],
        help='comma-separated sequence of glyph names to exclude.'
    )
    parser.add_argument(
        'input_path',
        metavar='FONT',
        help='OTF/TTF/WOFF/WOFF2 font file.',
    )
    options = parser.parse_args(args)

    options.font_paths_list = validate_font_paths([options.input_path])
    return options


def main(args=None):
    opts = get_options(args)

    if not opts.font_paths_list:
        return 1

    first_font_path = opts.font_paths_list[0]

    output_folder_path = get_output_folder_path(opts.output_folder_path,
                                                first_font_path)

    processFont(first_font_path, output_folder_path, opts)


if __name__ == "__main__":
    sys.exit(main())



# Copyright 2018 Adobe. All rights reserved.

import os
import sys
import unittest

from opentypesvg import addsvg, dumpsvg, fonts2svg

have_brotli = False
try:
    import brotli  # noqa

    have_brotli = True
except ImportError:
    pass

# addsvg must be last because of the way some tests prepare the input
ALL_TOOLS = (dumpsvg, fonts2svg, addsvg)
ROOT_DIR = os.path.dirname(os.path.dirname(__file__))
# addsvg requires a path to a folder in addition to the path to the font
XTRA_ARG = os.path.join(ROOT_DIR, 'fonts')


class OptionsTest(unittest.TestCase):

    def setUp(self):
        self.font_path = os.path.join(ROOT_DIR, 'fonts', 'Zebrawood.otf')

    # -----
    # Tests
    # -----

    def test_get_options_no_args(self):
        for tool in ALL_TOOLS:
            with self.assertRaises(SystemExit) as cm:
                tool.get_options([])
            self.assertEqual(cm.exception.code, 2)

    def test_get_options_help(self):
        for tool in ALL_TOOLS:
            with self.assertRaises(SystemExit) as cm:
                tool.get_options(['-h'])
            self.assertEqual(cm.exception.code, 0)

    def test_get_options_version(self):
        for tool in ALL_TOOLS:
            with self.assertRaises(SystemExit) as cm:
                tool.get_options(['--version'])
            self.assertEqual(cm.exception.code, 0)

    def test_get_options_bogus_option(self):
        args = ['--bogus', 'xfont']
        for tool in ALL_TOOLS:
            if tool is addsvg:
                args.insert(len(args) - 1, XTRA_ARG)  # insert the folder path
            with self.assertRaises(SystemExit) as cm:
                tool.get_options(args)
            self.assertEqual(cm.exception.code, 2)

    def test_get_options_invalid_font_path(self):
        args = ['xfont']
        for tool in ALL_TOOLS:
            if tool is addsvg:
                args.insert(len(args) - 1, XTRA_ARG)  # insert the folder path
            opts = tool.get_options(args)
            self.assertEqual(opts.font_paths_list, [])

    def test_get_options_not_a_font_path(self):
        args = [os.path.join('fonts', 'test.html')]
        for tool in ALL_TOOLS:
            if tool is addsvg:
                args.insert(len(args) - 1, XTRA_ARG)  # insert the folder path
            opts = tool.get_options(args)
            self.assertEqual(opts.font_paths_list, [])

    def test_get_options_good_font_path(self):
        args = [self.font_path]
        for tool in ALL_TOOLS:
            if tool is addsvg:
                args.insert(len(args) - 1, XTRA_ARG)  # insert the folder path
            opts = tool.get_options(args)
            self.assertEqual(os.path.basename(opts.font_paths_list[0]),
                             os.path.basename(self.font_path))

    def test_get_options_addsvg_invalid_folder_path(self):
        with self.assertRaises(SystemExit) as cm:
            addsvg.get_options(['-s', 'xfolder', 'xfont'])
        self.assertEqual(cm.exception.code, 1)

    @unittest.skipIf(have_brotli, "brotli module is installed")
    def test_get_options_addsvg_brotli_missing(self):
        with self.assertRaises(SystemExit) as cm:
            addsvg.get_options([XTRA_ARG, '-w', self.font_path])
        self.assertEqual(cm.exception.code, 1)

    @unittest.skipIf(not have_brotli, "brotli module is not installed")
    def test_get_options_addsvg_store_true_opts(self):
        args = [XTRA_ARG, '-w', '-z', self.font_path]
        opts = addsvg.get_options(args)
        attr = ('make_font_copy', 'strip_viewbox',
                'generate_woffs', 'compress_svgs')
        result = list(set([getattr(opts, name) for name in attr]))[0]
        self.assertTrue(result)

    def test_get_options_addsvg_store_false_opts(self):
        args = [XTRA_ARG, '-m', '-k', self.font_path]
        opts = addsvg.get_options(args)
        attr = ('make_font_copy', 'strip_viewbox',
                'generate_woffs', 'compress_svgs')
        result = list(set([getattr(opts, name) for name in attr]))[0]
        self.assertFalse(result)

    def test_get_options_dumpsvg_store_true_opts(self):
        args = ['-r', self.font_path]
        opts = dumpsvg.get_options(args)
        result = getattr(opts, 'reset_viewbox')
        self.assertTrue(result)

    def test_get_options_fonts2svg_store_true_opts(self):
        args = ['-u', self.font_path]
        opts = fonts2svg.get_options(args)
        result = getattr(opts, 'glyphsets_union')
        self.assertTrue(result)

    def test_get_options_addsvg_opts_defaults(self):
        dflt = {'make_font_copy': True,
                'strip_viewbox': True,
                'generate_woffs': False,
                'gnames_to_exclude': [],
                'compress_svgs': False}
        args = [XTRA_ARG, self.font_path]
        opts = addsvg.get_options(args)
        for key, val in dflt.items():
            self.assertEqual(getattr(opts, key), val)

    def test_get_options_dumpsvg_opts_defaults(self):
        dflt = {'output_folder_path': None,
                'reset_viewbox': False,
                'gnames_to_generate': [],
                'gnames_to_exclude': []}
        args = [self.font_path]
        opts = dumpsvg.get_options(args)
        for key, val in dflt.items():
            self.assertEqual(getattr(opts, key), val)

    def test_get_options_fonts2svg_opts_defaults(self):
        dflt = {'colors_list': [],
                'output_folder_path': None,
                'gnames_to_generate': [],
                'gnames_to_add': [],
                'gnames_to_exclude': [],
                'glyphsets_union': False}
        args = [self.font_path]
        opts = fonts2svg.get_options(args)
        for key, val in dflt.items():
            self.assertEqual(getattr(opts, key), val)

    def test_get_options_fonts2svg_invalid_hex_color(self):
        invalid_hex_colors = ['xxx', 'xxxxxx', 'aaa', 'aaaxxx']
        for hex_col in invalid_hex_colors:
            with self.assertRaises(SystemExit) as cm:
                fonts2svg.get_options(['-c', hex_col, self.font_path])
            self.assertEqual(cm.exception.code, 2)

    def test_get_options_fonts2svg_valid_hex_color(self):
        valid_hex_colors = ['aaabbb', 'cccdddee', '000000', 'ffffff00']
        for hex_col in valid_hex_colors:
            opts = fonts2svg.get_options(['-c', hex_col, self.font_path])
            self.assertEqual(opts.colors_list, [hex_col])

    def test_get_options_split_comma_sequence(self):
        invalid_comma_seqs = ['a,b', 'a, b', 'a ,b', 'a , b']
        for seq in invalid_comma_seqs:
            opts = fonts2svg.get_options(['-g', seq, self.font_path])
            self.assertEqual(opts.gnames_to_generate, ['a', 'b'])

    def test_get_options_addsvg_multiple_fonts(self):
        args = [XTRA_ARG, self.font_path, self.font_path]
        with self.assertRaises(SystemExit) as cm:
            addsvg.get_options(args)
        self.assertEqual(cm.exception.code, 2)

    def test_get_options_dumpsvg_multiple_fonts(self):
        args = [self.font_path, self.font_path]
        with self.assertRaises(SystemExit) as cm:
            dumpsvg.get_options(args)
        self.assertEqual(cm.exception.code, 2)

    def test_get_options_fonts2svg_multiple_fonts(self):
        args = [self.font_path, self.font_path, self.font_path]
        opts = fonts2svg.get_options(args)
        self.assertEqual(len(opts.font_paths_list), 3)

    def test_get_options_adjust_viewbox(self):
        opts = fonts2svg.get_options(['-av', 'xfont'])
        self.assertTrue(opts.adjust_view_box_to_glyph)

    def test_get_options_adjust_viewbox_2(self):
        opts = fonts2svg.get_options(['--adjust-viewbox', 'xfont'])
        self.assertTrue(opts.adjust_view_box_to_glyph)

    def test_get_options_adjust_viewbox_not_passed(self):
        opts = fonts2svg.get_options(['xfont'])
        self.assertFalse(opts.adjust_view_box_to_glyph)


if __name__ == "__main__":
    sys.exit(unittest.main())



# Copyright 2021 Adobe. All rights reserved.

import pytest

from opentypesvg.dumpsvg import main


def test_font_without_svg_table(base_font_path, capsys):
    with pytest.raises(SystemExit):
        main([base_font_path])
    captured = capsys.readouterr()
    assert captured.err == "ERROR: The font does not have the SVG table.\n"



# Copyright 2021 Adobe. All rights reserved.

from opentypesvg.fonts2svg import viewbox_settings


import os
import pytest


@pytest.mark.parametrize('file_name, expected', [
    ("1.ttf", "309 -1108 360 638"),
    ("2.ttf", "-904 -1110 416 650"),
    ("3.ttf", "-824 310 420 662"),
    ("4.ttf", "197 342 452 638"),
    ("12.ttf", "-904 -1110 1573 650"),
    ("13.ttf", "-824 -1108 1493 2080"),
    ("14.ttf", "197 -1108 472 2088"),
    ("23.ttf", "-904 -1110 500 2082"),
    ("24.ttf", "-904 -1110 1553 2090"),
    ("34.ttf", "-824 310 1473 670"),
    ("1234.ttf", "-904 -1110 1573 2090"),
])
def test_adjust_to_viewbox(file_name, expected, fixtures_dir):
    font_path = os.path.join(fixtures_dir, file_name)
    viewbox = viewbox_settings(font_path, True)
    assert viewbox == expected


def test_adjust_to_viewbox_default(base_font_path):
    viewbox = viewbox_settings(base_font_path, False)
    assert viewbox == "0 -1000 1000 1000"



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="opentypesvg",
    use_scm_version={'write_to': 'lib/opentypesvg/__version__.py'},
    setup_requires=["setuptools_scm"],
    author="Miguel Sousa",
    author_email="msousa@adobe.com",
    description="Tools for making OpenType-SVG fonts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/adobe-type-tools/opentype-svg",
    license="MIT",
    platforms=["Any"],
    package_dir={'': 'lib'},
    packages=['opentypesvg'],
    python_requires='>=3.6',
    install_requires=['fontTools[woff]>=3.1.0'],
    entry_points={
        'console_scripts': [
            "addsvg = opentypesvg.addsvg:main",
            "dumpsvg = opentypesvg.dumpsvg:main",
            "fonts2svg = opentypesvg.fonts2svg:main",
        ]
    },
)
